#pragma once

#include "Point.h"
#include "Image.h"

#include <memory>
#include <vector>
#include <string>

struct GlyphRunOutline
{
    bool mFilled{ true };
    std::vector<Point> mPoints;
};

class GlyphRunOutlines
{
public:
    void addToFeature(const Point& point);
    void startFeature(const Point& point, bool isFilled);

    const std::vector<GlyphRunOutline> getFeatures() const;

    std::string toPostScriptPath();
private:
    std::vector<GlyphRunOutline> mFeatures;
};

class FontGlyph
{

public:
    FontGlyph(unsigned width, unsigned height, int bearingX, int bearingY,
            int advanceX, std::unique_ptr<Image<unsigned char> > image);

    Image<unsigned char>* getImage() const;

    unsigned getWidth() const;
    unsigned getHeight() const;
    int getBearingX() const;
    int getBearingY() const;
    int getAdvanceX() const;

private:
    unsigned mWidth{0};
    unsigned mHeight{0};
    int mBearingX{0};
    int mBearingY{0};
    int mAdvanceX{0};
    std::unique_ptr<Image<unsigned char> > mImage;
};
