#include "DirectWriteFontEngine.h"

#include "FontGlyph.h"
#include "StringUtils.h"

#include "DirectWriteHelpers.h"

#include <dwrite.h>

void DirectWriteFontEngine::initialize()
{
	mIsValid = SUCCEEDED(::DWriteCreateFactory(DWRITE_FACTORY_TYPE_SHARED, __uuidof(IDWriteFactory), &mDWriteFactory));

    loadFontFaceFromName("Verdana", 16);
}

void DirectWriteFontEngine::initializeOffScreenRenderer()
{
    if (mOffScreenRenderer)
    {
        return;
    }
    auto pInterface = OffScreenTextRenderer::Create();
    pInterface->QueryInterface(IID_IOffScreenTextRenderer, &mOffScreenRenderer);
}

void DirectWriteFontEngine::loadFontFaceFromName(const std::string& fontName, int penSize)
{
    mDWriteFactory->CreateTextFormat(StringUtils::convert(fontName).c_str(), nullptr, DWRITE_FONT_WEIGHT_NORMAL, DWRITE_FONT_STYLE_NORMAL, DWRITE_FONT_STRETCH_NORMAL,
        penSize, L"en-us", &mTextFormat);
}

std::unique_ptr<FontGlyph> DirectWriteFontEngine::loadGlyph(uint32_t charCode)
{
	return nullptr;
}

std::unique_ptr<GlyphRunOutlines> DirectWriteFontEngine::getGlyphRunOutlines(const std::vector<uint32_t>& charCodes)
{
    initializeOffScreenRenderer();

    std::wstring text;
    for (auto code : charCodes)
    {
        text.push_back(code);
    }

    auto hr = mDWriteFactory->CreateTextLayout(text.c_str(), text.size(), mTextFormat.Get(), 50.0, 50.0, &mTextLayout);

    mTextLayout->Draw(nullptr, mOffScreenRenderer.Get(), 0.0, 0.0);

    return mOffScreenRenderer->getGlypyRunOutline();
}

