#include "DirectXPainter.h"

#include "DrawingContext.h"
#include "DrawingSurface.h"
#include "Scene.h"
#include "SceneItem.h"
#include "SceneModel.h"
#include "SceneText.h"

#include "TriMesh.h"

#include "FontsManager.h"
#include "FontGlyph.h"
#include "TextData.h"

#include "DirectXShaderProgram.h"
#include "DirectXMeshPainter.h"
#include "DirectXTextPainter.h"
#include "DirectXMesh.h"

#include "File.h"

#include <windows.h>

#include <iostream>

DirectXPainter::DirectXPainter(DrawingContext* context)
    :   AbstractPainter(context),
        mMeshPainter(std::make_unique<DirectXMeshPainter>()),
        mTextPainter(std::make_unique<DirectXTextPainter>())
{

}

DirectXMeshPainter* DirectXPainter::getMeshPainter() const
{
    return mMeshPainter.get();
}

DirectXTextPainter* DirectXPainter::getTextPainter() const
{
    return mTextPainter.get();
}

void DirectXPainter::initializeMesh(ID3D12Device* device)
{
    mMeshPainter->initializeD3d(device);
}

void DirectXPainter::initializeText(ID2D1DeviceContext2* d2dContext, IDWriteFactory* directWriteFactory)
{
    mTextPainter->initialize(d2dContext, directWriteFactory);
}

void DirectXPainter::updateMesh(ID3D12Device* device)
{
    mMeshPainter->updateBuffers(mDrawingContext, device);
}

void DirectXPainter::paint()
{
}

void DirectXPainter::paintBackground(const D3D12_CPU_DESCRIPTOR_HANDLE& rtvHandle, ID3D12GraphicsCommandList* commandList)
{
    const float clearColor[] = { 0.0f, 0.2f, 0.4f, 1.0f };
    commandList->ClearRenderTargetView(rtvHandle, clearColor, 0, nullptr);
}

void DirectXPainter::paintMesh(ID3D12GraphicsCommandList* commandList)
{
    mMeshPainter->updateCommandList(commandList);
}

void DirectXPainter::paintText(ID2D1DeviceContext2* d2dContext, IDWriteFactory* directWriteFactory)
{
    auto scene = mDrawingContext->getSurface()->getScene();
    for (const auto item : scene->getItems())
    {
        if (item->getType() == SceneItem::Type::TEXT && item->isVisible())
        {
            auto text = dynamic_cast<SceneText*>(item);
            mTextPainter->paint(text, mDrawingContext, d2dContext, directWriteFactory);
        }
    }
}
